unit ColorString;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Graphics, ExtCtrls;

type
  PColorString = ^TColorString;

  TColorString = record
    s: string;
    col: TColor;
  end;

  { TColorStringList }

  TColorStringList = class(TList)
  private
    function GetItem(index: integer): PColorString;
    procedure SetItem(index: integer; AValue: PColorString);
  public
    property Items[index: integer]: PColorString read GetItem write SetItem; default;
    destructor Destroy; override;

    function Add(s: string; col: TColor): integer;
  end;

  { TColorMemo }

  TColorMemo = class(TCustomPanel)
  public
    Lines: TColorStringList;
    constructor Create(TheOwner: TComponent); override;
    destructor Destroy; override;
  protected
    procedure Paint; override;
  end;

implementation

{ TColorMemo }

constructor TColorMemo.Create(TheOwner: TComponent);
begin
  inherited Create(TheOwner);
  Color := clBlack;

  Lines := TColorStringList.Create;
//  OnPaint := @Paint;
end;

destructor TColorMemo.Destroy;
begin
  Lines.Free;
  inherited Destroy;
end;

procedure TColorMemo.Paint;
var
  i: integer;
begin
  inherited Paint;
  for i := 0 to Lines.Count - 1 do begin
    Font.Color := Lines[i]^.col;
    Canvas.TextOut(0, i * 20, Lines[i]^.s);
  end;
end;

{ TColorStringList }

function TColorStringList.GetItem(index: integer): PColorString;
begin
  Result := PColorString(inherited Get(index));
end;

procedure TColorStringList.SetItem(index: integer; AValue: PColorString);
begin
  inherited Items[index] := AValue;
end;

destructor TColorStringList.Destroy;
var
  cs: PColorString;
  i: integer;
begin
  for i := 0 to Self.Count - 1 do begin
    cs := inherited Items[i];
    Dispose(cs);
  end;

  inherited Destroy;
end;

function TColorStringList.Add(s: string; col: TColor): integer;
var
  cs: PColorString;
begin
  New(cs);
  cs^.s := s;
  cs^.col := col;
  Result := inherited Add(cs);
end;

end.
